from ansible import errors

import re

def find_subnets(all_subnets, name, az=''):
    _list = []

    for subnet in all_subnets:
        if subnet['tags'].get('Name', '').startswith(name) and (az == '' or subnet["availability_zone"] == az):
                _list.append(subnet)
    return _list

class FilterModule(object):
    ''' A filter to split a string into a list. '''
    def filters(self):
        return {
            'find_subnets' : find_subnets
        }


#!/usr/bin/python
DOCUMENTATION = '''
module: cloudinit_iso
author:
    - "Moshe Immerman"
short_description:  Creates an ISO image containing cloud-init user-data files
description:
    -  Creates an ISO image containing cloud-init user-data files
options:
    dest:
        required: True
        description: ['']
    user:
        required: True
        description: ['']
    type:
        required: False
        default: vmware
        description: The type of cloud-init ISO to create, Valid values are: vmware,coreos
    meta:
        required: True
        description: ['']
'''
EXAMPLES = '''
     - cloudinit_iso:
          dest: /tmp/cloudinit.iso
          type: vmware
          user: |
            "
'''
import os.path
from subprocess import check_call
from ansible.module_utils.basic import AnsibleModule
import os, sys
import uuid
from tempfile import gettempdir

def log(s):
    sys.stderr.write("%s\n" % s)

def main():
    arg_spec = dict(
        dest=dict(required=True),
        user=dict(required=True),
        meta=dict(required=False),
        type=dict(required=False,default="vmware")
    )
    module = AnsibleModule(
        argument_spec=arg_spec,
        supports_check_mode=False
    )

    tmp = os.path.join(gettempdir(), '{}'.format(hash(os.times())))
    log("Writing temp user-data.txt: " + tmp)
    os.makedirs(tmp)

    path = "/user-data"
    if module.params['type'] == 'coreos':
        os.makedirs(tmp + "/openstack/latest")
        path = "/openstack/latest/user_data"

    with open(tmp + path, 'w') as f:
        f.write(module.params['user'])
    with open(tmp + "/meta-data", "w") as f:
        f.write('instance-id: i%s\n' % uuid.uuid1())

        if 'meta' in module.params and module.params['meta'] is not None:
            f.write(module.params['meta'])
        else:
            f.write('local-hostname: cloudy')

    if module.params['type'] == 'coreos':
       check_call("mkisofs -output %s -volid config-2 -joliet -rock ." % module.params['dest'], shell=True,cwd=tmp)
    else:
        check_call("mkisofs -output %s -volid cidata -joliet -rock user-data meta-data" % module.params['dest'], shell=True,cwd=tmp)

    module.exit_json(changed=True)


if __name__ == '__main__':
    main()


import re

def zone(hostname,count=2):
    i = int(re.findall('\d+$', hostname)[0])
    if i == 1:
        return 1
    elif i == 2:
        return 2
    elif i == 3:
        return 1
    elif i == 4:
        return 2


class FilterModule(object):
     def filters(self):
         return {'zone': zone}

if __name__ == '__main__':
    assert zone("AB8C01") == 1
    assert zone("ABC01") == 1
    assert zone("ABC02") == 2
    assert zone("ABC03") == 1
    assert zone("ABC01",2) == 1
    assert zone("ABC02",2) == 2
    assert zone("ABC03",2) == 1



from ansible import errors

import sys

def debug_obj(obj):
    sys.stderr.write(str(obj) + "\n")
    return ''

class FilterModule(object):

    def filters(self):
        return {
            'debug_obj' : debug_obj
        }


from ansible import errors

import re
import sys

def nested_element(object, path):
    obj = object
    for part in re.split("\\.|/", path):
        if part == "":
            continue
        if part not in obj:
            raise KeyError("Missing key %s in %s" % (part, obj))
        obj = obj[part]

    return obj

def walk_up(tree, path, ignore=[]):
    results = {}
    parts = re.split("\\.|/", path)
    for i in range(len(parts), 0, -1):
        subpath = "/".join(parts[0:i])
        object = nested_element(tree, subpath)
        for key in object:
            if key not in results and key not in ignore:
                results[key] = object[key]
    return results

class FilterModule(object):
    ''' Walks up an object tree from the lowest level collecting all attributes not available at lower levels'''
    def filters(self):
        return {
            'walk_up' : walk_up
        }


import os
import os.path

try:
    from __main__ import display
except ImportError:
    from ansible.utils.display import Display
    display = Display()


cwd = os.getcwd()

def hooks(hooks, type, run_once, groups):
  if isinstance(hooks, str):
    hooks = [hooks]
  suffix = "once.yml" if run_once else "yml"
  hooks.append(cwd)
  display.vv('Listing for: %s -> %s' % (hooks, groups))
  groups.insert(0,'all')

  files = []
  for hook in hooks:
    for group in groups:
      path = "%s/%s.%s.%s" % (hook, type, group, suffix)
      display.vv(path)
      if os.path.isfile(path):
        files.append(path)
  return files

class FilterModule(object):
  def filters(self):
      return {
          'deploy_hooks' : hooks
      }


#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright, (c) 2018, Ansible Project
# Copyright, (c) 2018, Moshe Immerman

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {
    'metadata_version': '1.1',
    'status': ['preview'],
    'supported_by': 'community'
}


DOCUMENTATION = '''
'''
EXAMPLES = '''
'''

RETURN = """
"""

try:
    from pyVmomi import vim
except ImportError:
    pass

from ansible.module_utils.basic import AnsibleModule
import os
import sys
import urllib3
import logging
import requests
import json
logger = logging.getLogger('vmware')
hdlr = logging.FileHandler('/tmp/ansible-vmware.log')
logger.addHandler(hdlr)
logger.setLevel(logging.INFO)
urllib3.disable_warnings()
from requests.auth import HTTPBasicAuth
headers = {
    "Content-Type": "application/json",
    "Accept": "application/json"
}



def to_dict(attributes):
    return {item['name'] : item['value'] for item in attributes}


class vRO:

    def __init__(self, host, username, password, verify=False):
        self.host = "https://" + host
        self.verify = verify
        self.username = username
        self.password = password

    def auth(self):
        return HTTPBasicAuth(self.username, self.password)

    def get(self, url, params={}):
        if not url.startswith('http'):
            url = self.host + "/" + url

        return requests.get(url,
            auth = self.auth(),
            params = params,
            verify = self.verify,
            headers = headers
        )

    def post(self, url, body):
        return requests.post(self.host + "/" + url,
            auth = self.auth(),
            data=json.dumps(body),
            verify = self.verify,
            headers = headers
        )

    def get_vcac_id(self):
        host = self.get("vco/api/catalog/vCAC/VCACHost").json()
        return to_dict(host['link'][0]['attributes'])['id']

    def get_reservations(self):
        return self.get('vco/api/inventory/vCAC/VCACHost/{id}/virtualFolder_Reservations/{id}%252FReservations/'.format(id=self.get_vcac_id()))

    def get_business_groups(self):
        return self.get('vco/api/inventory/vCAC/VCACHost/{id}/virtualFolder_ProvisioningGroups/{id}%252FProvisioningGroups/'.format(id=self.get_vcac_id()))


class Workflow:

    def __init__(self, r, name):
        self.r = r
        self.name = name
        self.results = None
        results = r.get("vco/api/workflows", {"conditions": "name=" + name})
        logger.info(str(results.status_code)  + "\n" + results.text)
        if results.status_code != 200:
            raise Exception(results.text)

        for attribute in json.loads(results.text)['link'][0]['attributes']:
            if attribute['name'] == 'id':
                self.id = attribute['value']
        results = r.get("vco/api/workflows/" + self.id)
        self.params = {}
        for param in  json.loads(results.text)["input-parameters"]:
            self.params[param['name']] = param

    def execute(self, params):
        if self.results != None:
            return self.results
        body = []
        for param in params:

            t = self.params[param]['type']
            p = { "scope": "local", "name": param, "type": t }
            if ":" not in t:
                value = {}
                value[t] = {
                    "value": params[param]
                }
                p['value'] = value

            else:
                p['value'] = {
                     "sdk-object" : {
                         "type" : t,
                          "id" :params[param]
                    }
                }

            body.append(p)
        res = self.r.post("vco/api/workflows/%s/executions" % self.id, {"parameters": body})
        if 'Location' in res.headers:
            res = self.r.get(res.headers['Location'])
        return res


def main():
    argument_spec = dict(
        hostname=dict(type='str', required=True),
        username=dict(type='str',required=True),
        password=dict(type='str',required=True,no_log=True),
        workflow=dict(type='str',required=True),
        params=dict(type='dict',required=True)
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=False
    )

    vro = vRO(module.params['hostname'],module.params['username'], module.params['password'])
    workflow = Workflow(vro, module.params['workflow'])
    logger.info("Executing %s with %s" % (workflow.name, module.params))
    result = workflow.execute(module.params['params'])
    logger.info("Executed %s with result =>  %s \n %s" % (workflow.name, result.status_code, result.text))
    if result.status_code == 200:
        module.exit_json(status_code=200, result=result.json())
    else:
        try:
            module.fail_json(msg=result.json()['message'], url=result.url)
        except:
             module.fail_json(msg="Unknown error executing workflow", response=response.text, url=result.url)


if __name__ == '__main__':
    main()




from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.vmware import PyVmomi
from ansible.module_utils.vmware_rest_client import VmwareRestClient
try:
    from com.vmware.cis.tagging_client import (Category, Tag, TagAssociation, CategoryModel)
    from com.vmware.vapi.std_client import DynamicID
except ImportError:
    pass

import logging

logger = logging.getLogger('vmware')
hdlr = logging.FileHandler('/tmp/ansible-vmware.log')
logger.addHandler(hdlr)
logger.setLevel(logging.INFO)

class VmwareTag(VmwareRestClient):
    def __init__(self, module):
        super(VmwareTag, self).__init__(module)
        self.tag_service = Tag(self.connect)
        self.tag_association = TagAssociation(self.connect)
        self.global_tags = dict()
        self.tag_name = self.params.get('tag_name')

        # for id in Category(self.connect).list():
        #     cat = Category(self.connect).get(id)
        #     logger.info(" \"%s\": %s" % (cat.name,id))


    def apply_tags(self, vm, category_ids, tags):

        dynamic_id = DynamicID(type='VirtualMachine', id=vm)
        attached = []

        for tag in self.tag_service.list():
            tag_obj = self.tag_service.get(tag)
            self.global_tags[tag_obj.category_id + tag_obj.name.lower()] = dict(tag_description=tag_obj.description,
                                                  tag_used_by=tag_obj.used_by,
                                                  tag_category_id=tag_obj.category_id,
                                                  tag_id=tag_obj.id
                                                  )

        logger.info(len(self.global_tags))
        for tag_id in self.tag_association.list_attached_tags(dynamic_id):
            attached.append(tag_id)

        for id in tags:
            key = category_ids[id] + tags[id].lower()
            if key not in self.global_tags:
                logger.info("Creating new tag: " + key)
                create_spec = self.tag_service.CreateSpec()
                create_spec.name = tags[id]
                create_spec.description = tags[id]
                create_spec.category_id = category_ids[id]
                tag = self.tag_service.create(create_spec)
            tag = self.global_tags[key]
            if tag['tag_id'] in attached:
                logger.info("Skipping " + id)
            else:
                logger.info("Applying %s (%s)=%s (%s) on %s" % (id,category_ids[id], tags[id],tag['tag_id'], vm))
                self.tag_association.attach(tag_id=tag['tag_id'], object_id=dynamic_id)

        for tag_id in self.tag_association.list_attached_tags(dynamic_id):
            attached.append(tag_id)
        return attached


def main():
    argument_spec = VmwareRestClient.vmware_client_argument_spec()
    argument_spec.update(
        vm=dict(type='str', required=True),
        uuid=dict(type='str', required=True),
        tags=dict(type='dict', required=True),
        category_ids=dict(type='dict', required=True),
    )
    module = AnsibleModule(argument_spec=argument_spec)
    pyv = PyVmomi(module)
    vm = pyv.get_vm()
    vmware_tag = VmwareTag(module)
    result = vmware_tag.apply_tags(vm._moId, module.params['category_ids'], module.params['tags'])
    module.exit_json()


if __name__ == '__main__':
    main()


from ansible import errors

def map_to_entries(_map,key, value):
    _list = []
    for k in _map:
        _list.append({
            key: k,
            value: _map[k]
        })
    return _list


class FilterModule(object):
    ''' A filter to convert a map into a list of entries'''
    def filters(self):
        return {
            'map_to_entries' : map_to_entries
        }

if __name__ == "__main__":
    print(map_to_entries({
        "key1": "value1",
        "key2": "value2"
        }, "k", "v"))




def to_map(_map, key, value):
    ret = {}
    for row in _map:
        ret[row[key].lower()] = row[value]
    return ret


class FilterModule(object):
     def filters(self):
         return {'to_map': to_map}




from ansible import errors

import re

def play_groups(play_hosts, groups, hostvars):
    _list = []

    for host in play_hosts:
        for group in groups:
            if host in groups[group]:
                _list.append(group)
    return list(set(_list))

class FilterModule(object):
    ''' A filter to split a string into a list. '''
    def filters(self):
        return {
            'play_groups' : play_groups
        }


from ansible import errors
import re

def from_si_unit(arg, base_uom='MB'):
  KB = 1024
  MB = KB * 1024
  GB = MB * 1024
  val = re.sub('[^0-9.]','', arg)
  uom =  re.sub('[0-9.]','', arg).upper()
  base = 0
  if uom.startswith('G'):
    base = int(float(val)) * GB
  elif uom.startswith('M'):
    base = int(float(val)) * MB
  elif uom.startswith('K'):
    base = int(float(val)) * KB
  else:
    base = int(float(val))

  if base_uom.startswith('M'):
    return base / MB
  elif base_uom.startswith('G'):
    return base / GB
  elif base_uom.startswith('K'):
    return base / KB
  return base



class FilterModule(object):
  '''Returns converts a string with a number and SI UOM to a number with a base of 1024, e.g. 1G == 1024'''
  def filters(self):
      return {
          'from_si_unit' : from_si_unit
      }

if __name__ == "__main__":
  print (from_si_unit('1.6G'))
  assert from_si_unit('1.6G') == 1638
  assert from_si_unit('1G') == 1024
  assert from_si_unit('1MiB') == 1
  assert from_si_unit('1M') == 1
  assert from_si_unit('1') == 1


#!/usr/bin/python

"""
Install Systemd units using Ansible
Handles unit restarting and systemd daemon reloading when the unit changes.
Also stops the unit correctly when uninstalled.

Install using pip install ansible-systemd-module

Example:

Use the normal systemd module to start it and enable it on boot
- name: Enable valu-backup service
  systemd: name="example.service" state=started enabled=yes
"""


DOCUMENTATION = '''
module: systemd_service
author:
    - "Moshe Immerman"
short_description:  Creates systemd services
description:
    -  Creates systemd services
options:
    Name:
        required: True
        description: ['']
    ExecStart:
        required: True
        description: ['']
    Description:
        required: false
        default: ''
        description: ['']
    RestartOn:
        required: false
        default: 'on-failure'
        description: ['']
    WantedBy:
        required: false
        default: 'multi-user.target'
        description: ['']
    RunAs:
        required: false
        default: 'root'
        description: ['']
    UnitArgs:
        required: false
        default: ''
        description: ['']
    ServiceArgs:
        required: false
        default: ''
        description: ['']
    InstallArgs:
        required: false
        default: ''
        description: ['']
    state:
        required: False
        default: present
        choices: ['present', 'absent']
        description: ['']
'''
EXAMPLES = '''
     - systemd_service:
          Name: test
          ExecStart: "/usr/bin/nc -l 200"
'''
import os.path
from subprocess import check_call
from ansible.module_utils.basic import AnsibleModule

import logging

logger = logging.getLogger('systemd')
hdlr = logging.FileHandler('/tmp/ansible-systemdunit.log')
logger.addHandler(hdlr)
logger.setLevel(logging.DEBUG)


ROOT = "/etc/systemd/system/"


def systemctl(arg1, arg2=None):
    if arg2 is None:
        check_call(["/bin/systemctl", arg1])
    else:
        check_call(["/bin/systemctl", arg1, arg2])
def present(unit_path, name, content):
    changed = False

    if not os.path.exists(unit_path):
        with open(unit_path, "w") as f:
            f.write(content)
            changed = True
        logger.info("Created new")
    else:
        current = open(unit_path).read()
        if current.strip() != content.strip():
            with open(unit_path, "w") as f:
                f.write(content)
                changed = True
            logger.info("Content changed")

    is_running = False

    try:
        systemctl("is-active", name)
        is_running = True
    except:
        pass

    if changed:
        systemctl("daemon-reload")

    if is_running and changed:
        logger.info("Restarting because changed and is running")
        systemctl("restart", name)

    return changed


def absent(unit_path, name):
    changed = False

    try:
        systemctl("stop", name)
        changed = True
        logger.info("Stopped")
    except:
        pass

    if os.path.exists(unit_path):
        check_call(["/bin/rm", "-f", unit_path])
        systemctl("daemon-reload")
        changed = True
        logger.info("Removed")

    return changed


def main():
    arg_spec = dict(
        Name=dict(required=True),
        Description=dict(default=None),
        ExecStart=dict(required=True),
        Restart=dict(default='on-failure'),
        WantedBy=dict(default='multi-user.target'),
        RunAs=dict(default=None),
        UnitArgs=dict(default=dict(), type='dict'),
        ServiceArgs=dict(default=dict(), type='dict'),
        InstallArgs=dict(default=dict(), type='dict'),
        state=dict(default='present', choices=['present', 'absent']),
    )
    module = AnsibleModule(
        argument_spec=arg_spec,
        supports_check_mode=False
    )

    name = module.params['Name'].strip()
    if name != name.lower():
        return module.fail_json(msg="systemd service names (%s) should be lowercase" % name)

    state = module.params['state']
    unit = module.params['UnitArgs']
    unit['Description'] = module.params['Description']
    service = module.params['ServiceArgs']
    service['ExecStart'] = module.params['ExecStart']
    service['RunAs'] = module.params['RunAs']
    service['Restart'] = module.params['Restart']
    install = module.params['InstallArgs']
    install['WantedBy'] = module.params['WantedBy']

    content = "[Unit]\n"
    for key in unit:
        if unit[key] != None:
            content += "%s=%s\n" % (key, unit[key])

    content += "\n[Service]\n"
    for key in service:
        if service[key] != None:
            content += "%s=%s\n" % (key, service[key])

    content += "\n[Install]\n"
    for key in install:
        if install[key] != None:
            content += "%s=%s\n" % (key, install[key])

    changed = False
    unit_path = ROOT + name + ".service"

    logger.info("Editing systemd unit " + name)

    if state == "present":
        changed = present(unit_path, name, content)
    elif state == "absent":
        changed = absent(unit_path, name)
    else:
        raise Exception("Unknown state param")

    module.exit_json(changed=changed)


if __name__ == '__main__':
    main()


import re

def zone(hostname,count=3,region=""):
    i = int(re.findall('\d+$', hostname)[0]) % count
    if region == "ap-northeast-2":
        return "a" if i % 2 == 1 else "c"
    if count == 1 or i == 1:
        return 'a'
    if i == 0:
        return chr(ord('a') + count-1)
    return chr(ord('a') + count -i)

class FilterModule(object):
     def filters(self):
         return {'zone': zone}

if __name__ == '__main__':
    assert zone("AB8C01") == 'a'
    assert zone("ABC01") == 'a'
    assert zone("ABC02") == 'b'
    assert zone("ABC03") == 'c'
    assert zone("ABC01",2) == 'a'
    assert zone("ABC02",2) == 'b'
    assert zone("ABC03",2) == 'a'
    assert zone("ABC01",2, "ap-northeast-2") == 'a'
    assert zone("ABC02",2, "ap-northeast-2") == 'c'
    assert zone("ABC03",2, "ap-northeast-2") == 'a'



from ansible import errors
import pdb

def reject_startswith(map, prefix):
    val = {}
    for key in map.keys():
        if not key.startswith(prefix):
            val[key] = map[key]
    return val


class FilterModule(object):
    def filters(self):
        return {
            'reject_startswith' : reject_startswith
        }


def parse_subnets(all_subnets):
    _list = {}

    for subnet in all_subnets:
        if subnet['tags']['Name'] not in _list:
            _list[subnet['tags']['Name']] = {}
        _list[subnet['tags']['Name']][subnet['availability_zone']] = subnet['id']
    return _list

class FilterModule(object):
    ''' A filter to split a string into a list. '''
    def filters(self):
        return {
            'parse_subnets' : parse_subnets
        }



def sub_map(map, prefix):
    ret = {}
    for key in map:
      if key.startswith(prefix):
        ret[key[len(prefix):]] = map[key]
    return ret


class FilterModule(object):
     def filters(self):
         """ Returns a new map/dict with only entries matching a prefix, and withthe prefix removed """
         return {'sub_map': sub_map}

if __name__ == "__main__":
    assert(sub_map({
        "elb.check": "/health",
        "elb.port": "100",
        "don.t": "match"
      }, "elb.") == {"check": "/health", "port": "100"})



from ansible import errors

import re

def nested_element(object, path):

    obj = object
    for part in re.split("\\.|/", path):
        if part == "":
            continue
        if part not in obj:
            raise KeyError("Missing key %s in %s" % (part, obj))
        obj = obj[part]

    return obj

class FilterModule(object):
    '''Returns an nested element from an object tree by path (seperated by / or .)'''
    def filters(self):
        return {
            'nested_element' : nested_element
        }


#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright, (c) 2018, Ansible Project
# Copyright, (c) 2018, Moshe Immerman
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {
    'metadata_version': '1.1',
    'status': ['preview'],
    'supported_by': 'community'
}


DOCUMENTATION = '''
'''

EXAMPLES = '''
'''

RETURN = """
"""

try:
    from pyVmomi import vim
except ImportError:
    pass

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.vmware import PyVmomi, vmware_argument_spec,find_cluster_by_name,wait_for_task


import logging

logger = logging.getLogger('vmware')
hdlr = logging.FileHandler('/tmp/ansible-vmware.log')
logger.addHandler(hdlr)
logger.setLevel(logging.INFO)


class VmGroupManager(PyVmomi):
    def __init__(self, module):
        super(VmGroupManager, self).__init__(module)

    def modify_groups(self, vm, groups):
        changed = False
        cluster = find_cluster_by_name(self.content, self.module.params['cluster'])

        for group in cluster.configurationEx.group:
          logger.info(group.name)
          for _group in groups:
            if group.name == _group:
             if vm not in group.vm:
                logger.info("Adding %s to %s", vm.name, group.vm)
                group.vm.append(vm)
                spec = vim.cluster.ConfigSpecEx(groupSpec=[vim.cluster.GroupSpec(info=group,operation="edit")])
                wait_for_task(cluster.ReconfigureEx(spec=spec, modify=True))
                changed = True

        return {'changed': changed, 'failed': False, 'groups': groups}



def main():
    argument_spec = vmware_argument_spec()
    argument_spec.update(
        datacenter=dict(type='str'),
        cluster=dict(type='str'),
        name=dict(required=False, type='str'),
        folder=dict(type='str'),
        uuid=dict(type='str'),
        groups=dict(
            type='list',
            default=[]
        ),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_one_of=[['name', 'uuid']],
    )

    if module.params.get('folder'):
        # FindByInventoryPath() does not require an absolute path
        # so we should leave the input folder path unmodified
        module.params['folder'] = module.params['folder'].rstrip('/')

    pyv = VmGroupManager(module)
    results = {'changed': False, 'failed': False, 'instance': dict()}

    # Check if the virtual machine exists before continuing
    vm = pyv.get_vm()

    if vm:
        results = pyv.modify_groups(vm, module.params['groups'])
        module.exit_json(**results)
    else:
        # virtual machine does not exists
        module.fail_json(msg="Unable to manage vm group membership for non-existing"
                             " virtual machine %s" % (module.params.get('name') or module.params.get('uuid')))


if __name__ == '__main__':
    main()



#!/usr/bin/python
# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = '''
'''

EXAMPLES = ''''''

import traceback

try:
    import boto3
    from botocore.exceptions import ClientError
    HAS_BOTO3 = True
except ImportError:
    HAS_BOTO3 = False

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.ec2 import (ansible_dict_to_boto3_filter_list,
                                      boto3_conn, boto3_tag_list_to_ansible_dict, camel_dict_to_snake_dict,
                                      ec2_argument_spec, get_aws_connection_info)


def get_instance_attribute(connection, module):

    instance_id = module.params.get("instance_id")
    attribute = module.params.get('attribute')
    response = connection.describe_instance_attribute(Attribute=attribute,InstanceId=instance_id)
    module.exit_json(attributes=response)


def main():

    argument_spec = ec2_argument_spec()
    argument_spec.update(
        dict(
            instance_id=dict(required=True),
            attribute=dict(required=True)
        )
    )

    module = AnsibleModule(argument_spec=argument_spec,
                           supports_check_mode=True
                           )

    if not HAS_BOTO3:
        module.fail_json(msg='boto3 required for this module')

    region, ec2_url, aws_connect_params = get_aws_connection_info(module, boto3=True)

    if region:
        connection = boto3_conn(module, conn_type='client', resource='ec2', region=region, endpoint=ec2_url, **aws_connect_params)
    else:
        module.fail_json(msg="region must be specified")

    get_instance_attribute(connection, module)


if __name__ == '__main__':
    main()


from ansible import errors
import os.path

def file_exists(file):
    return os.path.isfile(file)

def dir_exists(file):
    return os.path.isdir(file)

class FilterModule(object):
    '''Returns true if the file exists'''
    def filters(self):
        return {
            'file_exists' : file_exists,
            'dir_exists': dir_exists
        }



from subprocess import *
from setuptools import setup, find_packages
from setuptools.command.install import install
from ansible_provision import __version__
import os

__name__ = 'ansible-provision'

setup(
    name = __name__,
    version = __version__,
    scripts = ['ansible-provision'],
    install_requires = ['ansible-deploy'],
    packages = [__name__.replace("-", "_")],
    url = 'https://www/github.com/moshloop/ansible-provision',
    author = 'Moshe Immerman', author_email = 'moshe.immerman@gmail.com'
)


import re

def ebs_dev(index,count=3):
    index = int(index)
    if index == 0:
        return "/dev/xvdf"
    else:
        return "/dev/xvd" + chr(ord('f') + index)


class FilterModule(object):
     def filters(self):
         return {'ebs_dev': ebs_dev}

if __name__ == '__main__':
    assert ebs_dev("0") == '/dev/xvdf'
    assert ebs_dev("1") == '/dev/xvdg'



from ansible import errors

import re

def jsonpath(obj, expr):
    try:
        from jsonpath_rw import jsonpath, parse
    except:
        raise ModuleNotFoundError("Install jsonpath_rw using pip install jsonpath_rw first")
    return  parse(expr).find(obj)

class FilterModule(object):
    ''' A filter to transform data structures using jsonpath'''
    def filters(self):
        return {
            'jsonpath' : jsonpath
        }


from ansible import errors
import os.path

def is_empty(arg):
  if arg is None or str(arg).strip() is '':
    return True
  return False

class FilterModule(object):
  '''Returns true if the argument is null or an empty string'''
  def filters(self):
      return {
          'is_empty' : is_empty
      }



from ansible import errors

import re

def split_string(string, separator=' '):
    return string.split(separator)

def split_regex(string, separator_pattern='\s+'):
    return re.split(separator_pattern, string)

class FilterModule(object):
    ''' A filter to split a string into a list. '''
    def filters(self):
        return {
            'split' : split_string,
            'split_regex' : split_regex,
        }


#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright, (c) 2018, Ansible Project
# Copyright, (c) 2018, Moshe Immerman
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}


DOCUMENTATION = '''
---
module: vmware_guest_search
short_description: Search for VM's by name
description:
    -  Search for VM's by name
version_added: 2.8
author:
    - Moshe Immerman <moshe.immerman@gmail.com>
notes:
    - Tested on vSphere 6.5
requirements:
    - "python >= 2.6"
    - PyVmomi
options:
   name:
        description:
            - Name of the VM to search for.
        required: True
   datacenter:
         description:
            - Datacenter name where to search.
         required: True
   recursive:
        description:
            - Search for VM's recursively through all folders.
        default: False
        required: False
        type: bool
extends_documentation_fragment: vmware.documentation
'''

EXAMPLES = '''
- name: Search for all VM's starting with "base-vm"
  vmware_guest_search:
    hostname: '{{ vcenter_server }}'
    username: '{{ vcenter_username }}'
    password: '{{ vcenter_password }}'
    datacenter: '{{ vcenter_datacenter }}'
    validate_certs: no
    name: base-vm
  delegate_to: localhost
  register: facts
'''

RETURN = """
vms:
    description: A list of virtual machine names that start with the provided name
    returned: always
    type: list
"""

try:
    import pyVmomi
    from pyVmomi import vim
except ImportError:
    pass

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils._text import to_text
from ansible.module_utils.vmware import PyVmomi, vmware_argument_spec, find_datacenter_by_name


class PyVmomiHelper(PyVmomi):

    def __init__(self, module):
        super(PyVmomiHelper, self).__init__(module)

    def find_vms(self, folder, searchpath, recursive, vms):
        for vm in folder.childEntity:
            if vm.name.startswith(searchpath):
                vms.append(vm.name)
            if recursive and hasattr(vm, 'childEntity'):
                self.find_vms(vm, searchpath, recursive, vms)

    def find_folder(self, root, searchpath):
        """ Walk inventory objects one position of the searchpath at a time """
        # split the searchpath so we can iterate through it
        paths = [x.replace('/', '') for x in searchpath.split('/')]
        paths_total = len(paths) - 1
        position = 0

        # recursive walk while looking for next element in searchpath

        while root and position <= paths_total:
            change = False
            if hasattr(root, 'childEntity'):
                for child in root.childEntity:
                    if child.name == paths[position]:
                        root = child
                        position += 1
                        change = True
                        break
            elif isinstance(root, vim.Datacenter):
                if hasattr(root, 'vmFolder'):
                    if root.vmFolder.name == paths[position]:
                        root = root.vmFolder
                        position += 1
                        change = True
            else:
                root = None

            if not change:
                root = None

        return root


def main():
    argument_spec = vmware_argument_spec()
    argument_spec.update(
        name=dict(type='str', required=True),
        datacenter=dict(type='str', required=True),
        recursive=dict(type='bool', default=False)
    )
    module = AnsibleModule(argument_spec=argument_spec)
    try:
        pyv = PyVmomiHelper(module)
        dc = find_datacenter_by_name(pyv.content, module.params['datacenter'])
        if dc is None:
            module.fail_json(msg="Failed to find the datacenter %s" % module.params['datacenter'])
        folder = dc.vmFolder
        name = module.params['name']
        if "/" in name:
            paths = name.split("/")
            paths.pop()
            paths = "/".join(paths)
            name = name.split("/").pop()
            folder = pyv.find_folder(dc.vmFolder, paths)
            if folder is None:
                module.fail_json(msg="Unable to find folder: %s" % paths)
        vms = []
        pyv.find_vms(folder, name, module.params['recursive'], vms)
        vms.sort(reverse=True)
        module.exit_json(vms=vms)
    except Exception as exc:
        module.fail_json(msg="Fact gather failed with exception %s" % to_text(exc))


if __name__ == '__main__':
    main()
